module Search.Heuristic.PQueue where

import qualified Data.PSQueue as Q

class QueueClass q where
  null      :: (Ord k, Ord p) => q k p -> Bool
  empty     :: (Ord k, Ord p) => q k p
  singleton :: (Ord k, Ord p) => k -> p -> q k p
  singleton k p = insert k p $ empty
  delete    :: (Ord k, Ord p) => k -> q k p -> q k p
  deleteMin :: (Ord k, Ord p) => q k p -> q k p
  insert    :: (Ord k, Ord p) => k -> p -> q k p -> q k p
  findMin   :: (Ord k, Ord p) => q k p -> Maybe (k, p)
  lookup    :: (Ord k, Ord p) => k -> q k p -> Maybe p

type PQueue = Q.PSQ

instance QueueClass Q.PSQ where
  null      = Q.null
  empty     = Q.empty
  singleton = Q.singleton
  delete    = Q.delete
  deleteMin = Q.deleteMin
  insert    = Q.insert
  lookup    = Q.lookup
  findMin q = 
    case Q.findMin q of
      Nothing -> Nothing
      Just b -> Just ( Q.key b, Q.prio b )
